#include "task.h"
#include "applicationicon.h"
#include "aboutbox.h"
#include "menu.h"
#include "wimp_error.h"
#include "errorexception.h"

using namespace rov2;

namespace tutorial4
{
/* We can create a class like this... */
static CTask* pTask = new CTask("Fourth ROVLib2 app!");

/*! Defines my own menu */
class MyMenu : public CMenu
{
public:
  MyMenu() : CMenu("ROVLib2", 2)
  {
    m_pmiInfo = new CTextMenuItem("Info");
    m_pabBox = new CAboutBox("Fourth ROVLib2 app", "Test program", "Jason Tribbeck", "0.01 (02/09/2017)");
    m_pabBox->Create(tLocation{ 0, 0 });
    m_pmiInfo->AttachSubmenu(m_pabBox);
    m_pmiQuit = new CTextMenuItem("Quit");
    AddItem(m_pmiInfo);
    AddItem(m_pmiQuit);
  }

  eMenuHandled Clicked(const unsigned int* kuiaEntries)
  {
    switch (kuiaEntries[0])
    {
    case 1: // Second menu item
      pTask->Stop();
      return mhHandledClose;
    case 0:
      return mhHandledKeepOpen;
    }

    return mhUnhandled;
  }

private:
  CMenuItem* m_pmiQuit;
  CMenuItem* m_pmiInfo;
  CAboutBox* m_pabBox;
};

/*! Defines my own application icon */
class MyIcon : public CApplicationIcon
{
public:
  /*! We are going to use the "!system" sprite for our icon */
  MyIcon() : CApplicationIcon("!system")
  {
    m_mMyMenu = new MyMenu();
  }

  /*! Not much in the destructor! */
  ~MyIcon() {}

  bool Clicked(const tMouseClickBlock& rkmcbClickBlock)
  {
    if (rkmcbClickBlock.bcButtons.menu)
    {
      m_mMyMenu->DisplayAsIconbarMenu();
    }
    return true;
  }

private:
  CMenu* m_mMyMenu;
};
}

using namespace tutorial4;

int main(int argc, char** argv)
{
  try
  {
    /* Create our application icon */
    MyIcon* pIcon = new MyIcon();

    /* Poll until we want to stop*/
    pTask->Poll();

    /* Delete everything */
    delete(pIcon);
  }
  catch (_kernel_oserror* peb)
  {
    wimp_error(peb->errmess, WIMPERROR_CANCELBOX);
  }
  catch (CErrorException* exError)
  {
    wimp_error(exError->Message(), WIMPERROR_CANCELBOX);
  }

  return 0;
}
